/*
 * remote.cc: remote contol application for DVBs PCI card
 *
 * Copyright (C) 2000 Matjaz Thaler
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 
 * The author can be reached at matjaz.thaler@rd.iskraemeco.si 
 *
 * the project's page is at http://www2.arnes.si/~mthale1/index.html
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include <iostream.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include "channel.h"
#include "lirc_client.h"

#define irState shrMem[0]
#define irNumber shrMem[1]

struct satellite{
  char name[32];
  char file[32];
  int num;
};

struct satelliteList{
  int current;
  int max;
  struct satellite sat[32];
};

void SlaveTask();
void MasterTask();
void showOSD(char *text, int dev);
void closeOSD(int dev);
void channel_set(Channel &ch, char *buf,int dev);
void rotateDish(int num);
void sendIR(int num);
struct satelliteList readSatList();

char *progname;
char *shrMem;
char *irString;
int shmid;
struct shmid_ds shmbuf;

int main (int argc, char *argv[]){

  shmid = shmget(IPC_PRIVATE, 100, 0660);
  shrMem = (char *)shmat(shmid,NULL,SHM_RND);
  usleep(10000);
  irString = &shrMem[8]; 
  irState = 0;
  if(!fork()){
    shrMem = (char *)shmat(shmid,NULL,SHM_RND);
    usleep(10000);
    irString = &shrMem[8];
    SlaveTask();
    shmdt(shrMem);
    exit(0);
  }
  MasterTask();
  shmdt(shrMem);
  shmctl(shmid, IPC_RMID, &shmbuf);
  return(0);  
}

void SlaveTask(){
  char *buf;
  char remote[20];

  if(lirc_init("irexec")==-1) 
    exit(EXIT_FAILURE);  
  while((buf=lirc_nextir())!=NULL){
    while(irState != 0)
      usleep(50000);
    sscanf(buf, "%s %s %s %s", remote, remote, irString, remote);
    printf("lirc: remote: %s irString: %s\n", remote, irString);
    if(!strcmp(remote, "digital")){
      if(irString[1]==0x0 && irString[0]>0x2f && irString[0]<0x3a){
	irNumber=irString[0]-0x30;
	irState = 2;
      }
      else
	irState = 1;
    }
  }
}

void MasterTask(){
  FILE* fp;
  ChannelList chlist;
  int channel=0;
  int channelPrev=0;
  int videoDev;
  int timeOSD=0;
  int timeChann=0;
  int changeChannel=0;
  char tmpBuff[100];
  char *buff=tmpBuff;
  int i,j;
  int irLoop=0;
  char chanBuf[80];
  char *channelName=chanBuf;
  struct satelliteList satList;

  satList=readSatList();
  videoDev = open("/dev/video", O_RDWR);
  if(videoDev==-1){
    printf("could not open /dev/video\n");
    exit(1);
  }
  showOSD(satList.sat[satList.current].name, videoDev);
  chlist.init(satList.sat[satList.current].file);
  printf("Reading channel list from %s\n", satList.sat[satList.current].file);
  //rotateDish(satList.sat[satList.current].num);
  //sleep(2);
  channel_set(chlist(channel), channelName, videoDev);
  showOSD(channelName, videoDev);
  timeOSD=30;
  while(1){
    while(irState == 0){
      if(timeOSD > 0){
	if(--timeOSD == 0)
	  closeOSD(videoDev);
      }
      if(timeChann < 100)
	timeChann++;
      usleep(100000);
    }
    
    if(irState == 2){
      if(timeChann<12 && channel<100 && (10*channel+irNumber<chlist.NumChan()))
	channel = 10*channel+irNumber;
      else{
	channelPrev = channel;
	channel = irNumber;
      }	  
      changeChannel = 1;
      irState=0;
    }
    
    if(!strcmp(irString, "PREVIEW")){
      irState=0;
      i=3;
      sprintf(buff, " set preview time 1 .. 9 sec");
      showOSD(buff, videoDev);
      sleep(3);
      while(irState != 1){
	channelPrev=channel;
	channel++;
	if(channel >= chlist.NumChan()-1)
	  channel=0;
	channel_set(chlist(channel), channelName, videoDev);
	showOSD(channelName, videoDev);
	j=i+1;
	while(j>0){
	  j--;
	  sleep(1);
	  if(irState==2){
	    if(irNumber==0)
	      j=0;
	    else{
	      i=irNumber;
	      j=i;
	      sprintf(buff, " preview time is %d sec" ,i);
	      showOSD(buff, videoDev);
	    }
	    irState=0;
	  }
	  if(irState==1)
	    j=0;
	}
      }
      irState=0;
      showOSD(channelName, videoDev);
      timeOSD=30;
    }

    if(!strcmp(irString, "CHUP")){
      irState=0;
      channelPrev=channel;
      channel++;
      if(channel >= chlist.NumChan())
	channel=0;
      changeChannel = 1;
    }

    if(!strcmp(irString, "CHDOWN")){
      irState=0; 
      channelPrev=channel;
      channel--;
      if(channel < 0)
	channel=chlist.NumChan()-1;
      changeChannel = 1;
    }

    if(!strcmp(irString, "LOOP")){
      irState=0;
      i=channel;
      channel=channelPrev;
      channelPrev=i;
      changeChannel = 1;
    }
    
    if(!strcmp(irString, "DISPLAY")){
      irState=0;
      channelName[22]=0;
      sprintf(buff, "%s @ %s", channelName, satList.sat[satList.current].name);
      showOSD(buff, videoDev);
      timeOSD=30;
    }

    if(!strcmp(irString, "RIGHT")){
      irState=0;
      if(satList.current++==satList.max)
	satList.current=0;
      showOSD(satList.sat[satList.current].name, videoDev);
      chlist.init(satList.sat[satList.current].file);
      printf("Reading channel list from %s\n", satList.sat[satList.current].file);
      rotateDish(satList.sat[satList.current].num);
      sleep(1);
      if(irState == 0)
	sleep(2);
      channelPrev=0;
      channel=0;
      changeChannel = 1; 
    }

    if(!strcmp(irString, "LEFT")){
      irState=0;
      if(--satList.current<0)
	satList.current=satList.max;
      showOSD(satList.sat[satList.current].name, videoDev);
      chlist.init(satList.sat[satList.current].file);
      printf("Reading channel list from %s\n", satList.sat[satList.current].file);
      rotateDish(satList.sat[satList.current].num);
      sleep(1);
      if(irState == 0)
	sleep(2);
      channelPrev=0;
      channel=0;
      changeChannel = 1; 
    }


    if(changeChannel==1){
      channel_set(chlist(channel), channelName, videoDev);
      channelName[34]=0;
      timeChann=0;
      showOSD(channelName, videoDev);
      timeOSD=30;
      changeChannel = 0;  
    }
    if(irState == 0)
      irLoop=0;
    else{
      if(irLoop++ > 10){
	irState = 0;
	irLoop=0;
      }
    }
  }
  close(videoDev);
}

void showOSD(char *text, int dev){
  struct drawcmd dc;
  dc.cmd=OSD_Open;
  dc.x0=60;
  dc.y0=10;
  dc.x1=660;
  dc.y1=58;
  dc.color=1;
  ioctl(dev, VIDIOCSOSDCOMMAND, &dc);
  dc.cmd=OSD_Text;
  dc.x0=0;
  dc.y0=7;
  dc.x1=3; 
  dc.data=text;
  dc.color=3+(2<<16);
  ioctl(dev, VIDIOCSOSDCOMMAND, &dc);
}

void closeOSD(int dev){
  struct drawcmd dc;
  dc.cmd=OSD_Close;
  ioctl(dev, VIDIOCSOSDCOMMAND, &dc);
}

void channel_set(Channel &ch, char *buf,int dev){
  struct frontend front;
  uint freq;

  sprintf(buf, " %d  %s", ch.Num(), ch.Name());
  printf("Setting channel: %s\n", buf);
  ioctl(dev, VIDIOCGFRONTEND, &front);
  freq=(uint)ch.Freq()*1000UL;
  if (freq<11800000UL)
    front.freq=(freq -  9750000UL)*1000UL;
  else
    front.freq=(freq - 10600000UL)*1000UL; 
  front.ttk=(freq<11800000UL) ? 0 : 1;
  front.srate=ch.SR()*1000;
  front.fec=ch.FEC();
  front.video_pid=ch.V_pid();
  front.audio_pid=ch.A_pid();
  front.volt=ch.Pol()-9;
  front.diseqc=ch.Type()-10;
  ioctl(dev, VIDIOCSFRONTEND, &front);
}

void rotateDish(int num){
  int i, j;
  j=0;
  printf("Sending to analog receiver: %d\n", num);
  if(i=(int)(num/100)){
    num-=100*i;
    sendIR(i);
    j=1;
    if(num<10)
      sendIR(0);
  }
  if(i=(int)(num/10)){
    num-=10*i;
    sendIR(i);
  }
  sendIR(num);
}

void sendIR(int num){
  struct sockaddr_un addr;
  int fd;
  char buffer[32];
  
  usleep(200000);
  addr.sun_family=AF_UNIX;
  strcpy(addr.sun_path, LIRCD);
  fd=socket(AF_UNIX,SOCK_STREAM,0);
  connect(fd,(struct sockaddr *)&addr,sizeof(addr));
  sprintf(buffer,"SEND_ONCE analog %d\n", num);
  write(fd, (void *)buffer, strlen(buffer));
  close(fd);
}

struct satelliteList readSatList(){
  struct satelliteList satList;
  FILE* fp;
  int i,j;

  fp=fopen("xtvrc","r");
  if(fp==NULL){
    printf("could not open xtvrc\n");
    exit(1);
  }
  j=0;
  i=-1;
  while(j != -1){
    i++;
    fscanf(fp,"%s",satList.sat[i].name);
    if(satList.sat[i].name[0]=='*'){
      fscanf(fp,"%s",satList.sat[i].name);
      satList.current=i-1;
    }
    fscanf(fp,"%s",satList.sat[i].file);
    j=fscanf(fp,"%d",&satList.sat[i].num);
  }
  fclose(fp);
  satList.max=i-1;
  return satList;
}
